@@ -24,7 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../data/reference.h"
 #include "../output/output_format.h"
 #include "../output/output.h"
-#include "legacy/query_mapper.h"
+#include "legacy/pipeline.h"
 #include "../util/async_buffer.h"
 #include "../util/parallel/thread_pool.h"
 #if _MSC_FULL_VER == 191627042
@@ -315,7 +315,7 @@ void align_queries(Consumer* output_file, Search::Config& cfg)
 			message_stream << "Warning: resident size (" << (res_size + last_size) << ") exceeds memory limit." << std::endl;
 
 		timer.go("Sorting trace points");
-		ips4o::parallel::sort(hit_buf->data(), hit_buf->data() + hit_buf->size(), std::less<>(), config.threads_);
+		ips4o::parallel::sort(hit_buf->begin(), hit_buf->end(), std::less<Search::Hit>(), config.threads_);
 		statistics.inc(Statistics::TIME_SORT_SEED_HITS, timer.microseconds());
 
 #ifndef OLD
