@@ -415,6 +415,9 @@ struct HspContext
 	int reserved2() const {
 		return hsp_.reserved2;
 	}
+	Hsp hsp() const {
+		return hsp_;
+	}
 	HspContext& parse(const OutputFormat* output_format);
 
 	TranslatedSequence query;
@@ -426,7 +429,7 @@ struct HspContext
 	int ungapped_score;
 	double query_self_aln_score, target_self_aln_score;
 	Sequence subject_seq;
-private:	
+private:
 	Hsp hsp_;
 	template<typename T> friend struct TypeDeserializer;
 };
@@ -448,6 +451,7 @@ struct TypeSerializer<HspContext> {
 		buf_->write(h.subject_len);
 		buf_->write(h.identities());
 		buf_->write(h.mismatches());
+		buf_->write(h.positives());
 		buf_->write(h.gaps());
 		buf_->write(h.length());
 		buf_->write(h.gap_openings());
@@ -485,6 +489,7 @@ struct TypeDeserializer<HspContext> {
 		file_->read(h.subject_len);
 		file_->read(h.hsp_.identities);
 		file_->read(h.hsp_.mismatches);
+		file_->read(h.hsp_.positives);
 		file_->read(h.hsp_.gaps);
 		file_->read(h.hsp_.length);
 		file_->read(h.hsp_.gap_openings);
