@@ -20,18 +20,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #pragma once
-#include <atomic>
-#include "../basic/const.h"
-#include "../util/hash_table.h"
-#include "seed_array.h"
+#include "flags.h"
 #include "../util/algo/join_result.h"
 #include "../util/range.h"
+#include "../run/config.h"
+#include "../util/data_structures/double_array.h"
+#include "seed_histogram.h"
+
+struct SequenceSet;
 
 struct Frequent_seeds
 {
 
-	void build(unsigned sid, const SeedPartitionRange &range, DoubleArray<SeedArray::_pos> *query_seed_hits, DoubleArray<SeedArray::_pos> *ref_seed_hits);
-	static void clear_masking(Sequence_set& seqs);
+	void build(unsigned sid, const SeedPartitionRange &range, DoubleArray<SeedLoc> *query_seed_hits, DoubleArray<SeedLoc> *ref_seed_hits, Search::Config& cfg);
+	static void clear_masking(SequenceSet& seqs);
 
 private:
 
@@ -40,17 +42,15 @@ private:
 	static void build_worker(
 		size_t seedp,
 		size_t thread_id,
-		DoubleArray<SeedArray::_pos> *query_seed_hits,
-		DoubleArray<SeedArray::_pos> *ref_seed_hits,
+		DoubleArray<SeedLoc> *query_seed_hits,
+		DoubleArray<SeedLoc> *ref_seed_hits,
 		const SeedPartitionRange *range,
 		unsigned sid,
 		unsigned ref_max_n,
 		unsigned query_max_n,
-		vector<unsigned> *counts);
-
-	static void compute_sd(std::atomic<unsigned> *seedp, DoubleArray<SeedArray::_pos> *query_seed_hits, DoubleArray<SeedArray::_pos> *ref_seed_hits, vector<Sd> *ref_out, vector<Sd> *query_out);
+		std::vector<unsigned> *counts,
+		Search::Config* cfg);
 
 };
 
-extern Frequent_seeds frequent_seeds;
-
+extern Frequent_seeds frequent_seeds;
\ No newline at end of file
