@@ -26,8 +26,5 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "sequence_file.h"
 #include "seed_histogram.h"
 
-extern int64_t current_ref_block;
-extern bool blocked_processing;
-
-Chunk to_chunk(const string& line);
-string to_string(const Chunk& c);
\ No newline at end of file
+Chunk to_chunk(const std::string& line);
+std::string to_string(const Chunk& c);
\ No newline at end of file
