@@ -69,9 +69,9 @@ struct BufferedWriter
 				flush(p);
 		}
 	}
-	void flush(unsigned p)
+	NO_INLINE void flush(unsigned p)
 	{
-		memcpy(ptr[p], buf[p], n[p] * sizeof(SeedArray::Entry));
+		memcpy(ptr[p], buf[p], (size_t)n[p] * sizeof(SeedArray::Entry));
 		ptr[p] += n[p];
 		n[p] = 0;
 	}
@@ -160,7 +160,7 @@ struct BufferedWriter2
 				flush(p);
 		}
 	}
-	void flush(unsigned p)
+	NO_INLINE void flush(unsigned p)
 	{
 		out[p].push_back(buf[p], n[p]);
 		n[p] = 0;
