@@ -33,9 +33,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 using std::set;
 using std::endl;
 using std::make_pair;
+using std::vector;
+using std::pair;
+using std::string;
 
 TaxonList::TaxonList(Deserializer &in, size_t size, size_t data_size):
-	CompactArray<vector<uint32_t>>(in, size, data_size)
+	CompactArray<vector<TaxId>>(in, size, data_size)
 {}
 
 static int mapping_file_format(const string& header) {
@@ -52,9 +55,9 @@ static int mapping_file_format(const string& header) {
 	throw std::runtime_error("Accession mapping file header has to be in one of these formats:\naccession\taccession.version\ttaxid\tgi\naccession.version\ttaxid");
 }
 
-static void load_mapping_file(ExternalSorter<pair<string, uint32_t>>& sorter)
+static void load_mapping_file(ExternalSorter<pair<string, TaxId>>& sorter)
 {
-	unsigned taxid;
+	TaxId taxid;
 	TextInputFile f(config.prot_accession2taxid);
 	f.getline();
 	int format = mapping_file_format(f.line);
@@ -78,29 +81,27 @@ static void load_mapping_file(ExternalSorter<pair<string, uint32_t>>& sorter)
 			accession.erase(i);
 
 		if (accession != last)
-			sorter.push(make_pair(accession, (uint32_t)taxid));
+			sorter.push(make_pair(accession, taxid));
 
 		last = accession;
 	}
 	f.close();
 }
 
-void TaxonList::build(OutputFile &db, ExternalSorter<pair<string, uint32_t>>& acc2oid, size_t seqs, Table& stats)
+void TaxonList::build(OutputFile &db, ExternalSorter<pair<string, OId>>& acc2oid, OId seqs, Util::Table& stats)
 {
-	typedef pair<string, uint32_t> T;
-
 	task_timer timer("Loading taxonomy mapping file");
-	ExternalSorter<T> acc2taxid;
+	ExternalSorter<pair<string, TaxId>> acc2taxid;
 	load_mapping_file(acc2taxid);
 
 	timer.go("Joining accession mapping");
 	acc2taxid.init_read();
 	acc2oid.init_read();
-	const auto cmp = [](const T& x, const T& y) { return x.first < y.first; };
-	const auto value = [](const T& x, const T& y) { return make_pair(x.second, y.second); };
-	auto it = join_sorted_lists(acc2oid, acc2taxid, cmp, value);
+	const auto cmp = [](const pair<string, OId>& x, const pair<string, TaxId>& y) { return x.first < y.first; };
+	const auto value = [](const pair<string, OId>& x, const pair<string, TaxId>& y) { return make_pair(x.second, y.second); };
+	auto it = join_sorted_lists(acc2oid, acc2taxid, First<string, OId>(), First<string, TaxId>(), value);
 
-	ExternalSorter<pair<uint32_t, uint32_t>> oid2taxid;
+	ExternalSorter<pair<OId, TaxId>> oid2taxid;
 	size_t acc_matched = 0;
 	while (it.good()) {
 		oid2taxid.push(*it);
@@ -111,11 +112,10 @@ void TaxonList::build(OutputFile &db, ExternalSorter<pair<string, uint32_t>>& ac
 	timer.go("Writing taxon id list");
 	db.set(Serializer::VARINT);
 	oid2taxid.init_read();
-	const uint32_t n = (uint32_t)seqs;
-	auto taxid_it = merge_keys(oid2taxid, First<uint32_t, uint32_t>(), Second<uint32_t, uint32_t>(), 0);
+	auto taxid_it = merge_keys(oid2taxid, First<OId, TaxId>(), Second<OId, TaxId>(), 0);
 	size_t mapped_seqs = 0;
-	while (taxid_it.key() < n) {
-		set<uint32_t> tax_ids = *taxid_it;
+	while (taxid_it.key() < seqs) {
+		set<TaxId> tax_ids = *taxid_it;
 		tax_ids.erase(0);
 		db << tax_ids;
 		++taxid_it;
