@@ -30,6 +30,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../data/sequence_set.h"
 #include "../stats/cbs.h"
 #include "flags.h"
+#include "../util/parallel/thread_pool.h"
 
 struct DpTarget
 {
@@ -42,7 +43,7 @@ struct DpTarget
 		{}
 		int i1, j1, ident, len;
 	};
-	enum { BLANK = -1 };
+	enum { BLANK = -1, MIN_LETTERS = 3 };
 	static Loc banded_cols(const Loc qlen, const Loc tlen, const Loc d_begin, const Loc d_end) {
 		const Loc pos = std::max(d_end - 1, 0) - (d_end - 1);
 		const Loc d0 = d_begin;
@@ -56,23 +57,27 @@ struct DpTarget
 		target_idx(BLANK),
 		matrix(nullptr)
 	{}
-	DpTarget(const Sequence &seq, int true_target_len, int d_begin, int d_end, int target_idx, int qlen, const Stats::TargetMatrix* matrix = nullptr, const CarryOver& carry_over = CarryOver()) :
+	DpTarget(const Sequence &seq, int true_target_len, int d_begin, int d_end, Interval chaining_target_range, Score chaining_score, BlockId target_idx, int qlen, const Stats::TargetMatrix* matrix = nullptr, const CarryOver& carry_over = CarryOver(), const Anchor& anchor = Anchor()) :
 		seq(seq),
 		d_begin(d_begin),
 		d_end(d_end),
 		cols(banded_cols(qlen, seq.length(), d_begin, d_end)),
 		true_target_len(true_target_len),
+		chaining_target_range(chaining_target_range),
+		chaining_score(chaining_score),
 		target_idx(target_idx),
 		carry_over(carry_over),
-		matrix(matrix)
+		matrix(matrix),
+		anchor(anchor)
 	{
 	}
-	DpTarget(const Sequence& seq, int true_target_len, int target_idx, const Stats::TargetMatrix* matrix = nullptr, const CarryOver& carry_over = CarryOver()):
+	DpTarget(const Sequence& seq, int true_target_len, BlockId target_idx, const Stats::TargetMatrix* matrix = nullptr, const CarryOver& carry_over = CarryOver()):
 		seq(seq),
 		d_begin(),
 		d_end(),
 		cols(),
 		true_target_len(true_target_len),
+		chaining_score(0),
 		target_idx(target_idx),
 		carry_over(carry_over),
 		matrix(matrix)
@@ -110,11 +115,30 @@ struct DpTarget
 	int matrix_scale() const {
 		return adjusted_matrix() ? config.cbs_matrix_scale : 1;
 	}
+	int64_t cells(DP::Flags flags, Loc qlen) const {
+		return flag_any(flags, DP::Flags::FULL_MATRIX) ? (int64_t)seq.length() * (int64_t)qlen
+			: int64_t(d_end - d_begin) * (int64_t)cols;
+	}
+	bool extend_right(Loc qlen) const {
+		//return anchor.query_end() < qlen && anchor.subject_end() < seq.length()
+		return std::min(qlen - anchor.query_end(), seq.length() - anchor.subject_end()) >= MIN_LETTERS;
+	}
+	bool extend_left() const {
+		//return anchor.query_begin() > 0 && anchor.subject_begin() > 0
+		return std::min(anchor.query_begin(), anchor.subject_begin()) >= MIN_LETTERS;
+	}
+	friend std::ostream& operator<<(std::ostream& s, const DpTarget& t) {
+		s << t.seq << '\t' << t.d_begin << '\t' << t.d_end << '\t' << t.cols << '\t' << t.true_target_len << '\t' << t.target_idx;
+		return s;
+	}
 	Sequence seq;
 	Loc d_begin, d_end, cols, true_target_len;
-	int target_idx;
+	Interval chaining_target_range;
+	Score chaining_score;
+	BlockId target_idx;
 	CarryOver carry_over;
 	const Stats::TargetMatrix* matrix;
+	Anchor anchor;
 };
 
 struct DpStat
@@ -142,12 +166,14 @@ namespace DP {
 
 struct Params {
 	const Sequence query;
+	const char* query_id;
 	const Frame frame;
 	const int query_source_len;
 	const int8_t* const composition_bias;
 	const Flags flags;
-	const HspValues v;
+	HspValues v;
 	Statistics& stat;
+	ThreadPool* thread_pool;
 };
 
 enum { BINS = 6, SCORE_BINS = 3, ALGO_BINS = 2 };
@@ -160,6 +186,18 @@ using Targets = std::array<std::vector<DpTarget>, BINS>;
 struct NoCBS {
 	constexpr void* operator[](int i) const { return nullptr; }
 };
+
+namespace AnchoredSwipe {
+
+struct Config {
+	Sequence query;
+	const int8_t* query_cbs;
+	Score score_hint;
+	Statistics& stats;
+	ThreadPool* thread_pool;
+};
+
+}
 	
 namespace Swipe {
 
@@ -172,9 +210,10 @@ namespace BandedSwipe {
 DECL_DISPATCH(std::list<Hsp>, swipe, (const Targets& targets, Params& params))
 DECL_DISPATCH(std::list<Hsp>, swipe_set, (const SequenceSet::ConstIterator begin, const SequenceSet::ConstIterator end, Params& params))
 DECL_DISPATCH(unsigned, bin, (HspValues v, int query_len, int score, int ungapped_score, const int64_t dp_size, unsigned score_width, const Loc mismatch_est))
+DECL_DISPATCH(std::list<Hsp>, anchored_swipe, (Targets& targets, const DP::AnchoredSwipe::Config& cfg))
 
 }
 
 }
 
-DECL_DISPATCH(std::list<Hsp>, banded_3frame_swipe, (const TranslatedSequence &query, Strand strand, vector<DpTarget>::iterator target_begin, vector<DpTarget>::iterator target_end, DpStat &stat, bool score_only, bool parallel))
+DECL_DISPATCH(std::list<Hsp>, banded_3frame_swipe, (const TranslatedSequence &query, Strand strand, std::vector<DpTarget>::iterator target_begin, std::vector<DpTarget>::iterator target_end, DpStat &stat, bool score_only, bool parallel))
