@@ -1,6 +1,6 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2021 Max Planck Society for the Advancement of Science e.V.
+Copyright (C) 2013-2022 Max Planck Society for the Advancement of Science e.V.
                         Benjamin Buchfink
                         Eberhard Karls Universitaet Tuebingen
 						
@@ -25,14 +25,197 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 namespace DISPATCH_ARCH {
 
-#if ARCH_ID == 2
+#if ARCH_ID == 3
 
 template<int DELTA>
 struct ScoreVector<int8_t, DELTA>
 {
 
 	ScoreVector() :
-		data_(::SIMD::_mm256_set1_epi8(DELTA))
+		data_(_mm512_set1_epi8(DELTA))
+	{}
+
+	explicit ScoreVector(__m512i data) :
+		data_(data)
+	{}
+
+	explicit ScoreVector(int8_t x) :
+		data_(_mm512_set1_epi8(x))
+	{}
+
+	explicit ScoreVector(int x) :
+		data_(_mm512_set1_epi8(x))
+	{}
+
+	explicit ScoreVector(const int8_t* s) :
+		data_(_mm512_loadu_si512(reinterpret_cast<const __m512i*>(s)))
+	{ }
+
+	explicit ScoreVector(const uint8_t* s) :
+		data_(_mm512_loadu_si512(reinterpret_cast<const __m512i*>(s)))
+	{ }
+
+	ScoreVector(unsigned a, __m512i seq)
+	{
+		/*const __m256i* row_lo = reinterpret_cast<const __m256i*>(&score_matrix.matrix8_low()[a << 5]);
+		const __m256i* row_hi = reinterpret_cast<const __m256i*>(&score_matrix.matrix8_high()[a << 5]);
+
+		__m256i high_mask = _mm256_slli_epi16(_mm256_and_si256(seq, _mm256_set1_epi8('\x10')), 3);
+		__m256i seq_low = _mm256_or_si256(seq, high_mask);
+		__m256i seq_high = _mm256_or_si256(seq, _mm256_xor_si256(high_mask, _mm256_set1_epi8('\x80')));
+
+		__m256i r1 = _mm512_load_si512(row_lo);
+		__m256i r2 = _mm512_load_si512(row_hi);
+
+		__m256i s1 = _mm256_shuffle_epi8(r1, seq_low);
+		__m256i s2 = _mm256_shuffle_epi8(r2, seq_high);
+		data_ = _mm256_or_si256(s1, s2);*/
+	}
+
+	ScoreVector operator+(const ScoreVector& rhs) const
+	{
+		return ScoreVector(_mm512_adds_epi8(data_, rhs.data_));
+	}
+
+	ScoreVector operator-(const ScoreVector& rhs) const
+	{
+		return ScoreVector(_mm512_subs_epi8(data_, rhs.data_));
+	}
+
+	ScoreVector& operator+=(const ScoreVector& rhs) {
+		data_ = _mm512_adds_epi8(data_, rhs.data_);
+		return *this;
+	}
+
+	ScoreVector& operator-=(const ScoreVector& rhs)
+	{
+		data_ = _mm512_subs_epi8(data_, rhs.data_);
+		return *this;
+	}
+
+	ScoreVector& operator &=(const ScoreVector& rhs) {
+		data_ = _mm512_and_si512(data_, rhs.data_);
+		return *this;
+	}
+
+	ScoreVector& operator++() {
+		data_ = _mm512_adds_epi8(data_, _mm512_set1_epi8(1));
+		return *this;
+	}
+
+	friend ScoreVector blend(const ScoreVector&v, const ScoreVector&w, const ScoreVector&mask) {
+		return ScoreVector(); // (_mm256_blendv_epi8(v.data_, w.data_, mask.data_));
+	}
+
+	ScoreVector operator==(const ScoreVector&v) const {
+		return ScoreVector(); // ScoreVector(_mm256_cmpeq_epi8(data_, v.data_));
+	}
+
+	friend uint32_t cmp_mask(const ScoreVector&v, const ScoreVector&w) {
+		return 0; // (uint32_t)_mm256_movemask_epi8(_mm256_cmpeq_epi8(v.data_, w.data_));
+	}
+
+	int operator [](unsigned i) const
+	{
+		return *(((uint8_t*)&data_) + i);
+	}
+
+	ScoreVector& set(unsigned i, uint8_t v)
+	{
+		*(((uint8_t*)&data_) + i) = v;
+		return *this;
+	}
+
+	ScoreVector& max(const ScoreVector& rhs)
+	{
+		data_ = _mm512_max_epi8(data_, rhs.data_);
+		return *this;
+	}
+
+	ScoreVector& min(const ScoreVector& rhs)
+	{
+		data_ = _mm512_min_epi8(data_, rhs.data_);
+		return *this;
+	}
+
+	friend ScoreVector max(const ScoreVector& lhs, const ScoreVector& rhs)
+	{
+		return ScoreVector(_mm512_max_epi8(lhs.data_, rhs.data_));
+	}
+
+	friend ScoreVector min(const ScoreVector& lhs, const ScoreVector& rhs)
+	{
+		return ScoreVector(_mm512_min_epi8(lhs.data_, rhs.data_));
+	}
+
+	void store(int8_t* ptr) const
+	{
+		_mm512_storeu_si512((__m512i*)ptr, data_);
+	}
+
+	friend std::ostream& operator<<(std::ostream& s, ScoreVector v)
+	{
+		int8_t x[32];
+		v.store(x);
+		for (unsigned i = 0; i < 32; ++i)
+			printf("%3i ", (int)x[i]);
+		return s;
+	}
+
+	void expand_from_8bit() {}
+
+	__m512i data_;
+
+};
+
+template<int DELTA>
+struct ScoreTraits<ScoreVector<int8_t, DELTA>>
+{
+	enum { CHANNELS = 64 };
+	typedef ::DISPATCH_ARCH::SIMD::Vector<int8_t> Vector;
+	typedef int8_t Score;
+	typedef uint8_t Unsigned;
+	typedef uint32_t Mask;
+	struct TraceMask {
+		static uint64_t make(uint32_t vmask, uint32_t hmask) {
+			return (uint64_t)vmask << 32 | (uint64_t)hmask;
+		}
+		static uint64_t vmask(int channel) {
+			return (uint64_t)1 << (channel + 32);
+		}
+		static uint64_t hmask(int channel) {
+			return (uint64_t)1 << channel;
+		}
+		uint64_t gap;
+		uint64_t open;
+	};
+	static ScoreVector<int8_t, DELTA> zero() {
+		return ScoreVector<int8_t, DELTA>();
+	}
+	static constexpr int8_t max_score() {
+		return SCHAR_MAX;
+	}
+	static int int_score(int8_t s)
+	{
+		return (int)s - DELTA;
+	}
+	static constexpr int max_int_score() {
+		return SCHAR_MAX - DELTA;
+	}
+	static constexpr int8_t zero_score() {
+		return DELTA;
+	}
+	static void saturate(ScoreVector<int8_t, DELTA>& v) {}
+};
+
+#elif ARCH_ID == 2
+
+template<int DELTA>
+struct ScoreVector<int8_t, DELTA>
+{
+
+	ScoreVector() :
+		data_(_mm256_set1_epi8(DELTA))
 	{}
 
 	explicit ScoreVector(__m256i data) :
@@ -40,11 +223,11 @@ struct ScoreVector<int8_t, DELTA>
 	{}
 
 	explicit ScoreVector(int8_t x) :
-		data_(::SIMD::_mm256_set1_epi8(x))
+		data_(_mm256_set1_epi8(x))
 	{}
 
 	explicit ScoreVector(int x) :
-		data_(::SIMD::_mm256_set1_epi8(x))
+		data_(_mm256_set1_epi8(x))
 	{}
 
 	explicit ScoreVector(const int8_t* s) :
@@ -60,9 +243,11 @@ struct ScoreVector<int8_t, DELTA>
 		const __m256i* row_lo = reinterpret_cast<const __m256i*>(&score_matrix.matrix8_low()[a << 5]);
 		const __m256i* row_hi = reinterpret_cast<const __m256i*>(&score_matrix.matrix8_high()[a << 5]);
 
-		__m256i high_mask = _mm256_slli_epi16(_mm256_and_si256(seq, ::SIMD::_mm256_set1_epi8('\x10')), 3);
+		seq = letter_mask(seq);
+
+		__m256i high_mask = _mm256_slli_epi16(_mm256_and_si256(seq, _mm256_set1_epi8('\x10')), 3);
 		__m256i seq_low = _mm256_or_si256(seq, high_mask);
-		__m256i seq_high = _mm256_or_si256(seq, _mm256_xor_si256(high_mask, ::SIMD::_mm256_set1_epi8('\x80')));
+		__m256i seq_high = _mm256_or_si256(seq, _mm256_xor_si256(high_mask, _mm256_set1_epi8('\x80')));
 
 		__m256i r1 = _mm256_load_si256(row_lo);
 		__m256i r2 = _mm256_load_si256(row_hi);
@@ -99,7 +284,7 @@ struct ScoreVector<int8_t, DELTA>
 	}
 
 	ScoreVector& operator++() {
-		data_ = _mm256_adds_epi8(data_, ::SIMD::_mm256_set1_epi8(1));
+		data_ = _mm256_adds_epi8(data_, _mm256_set1_epi8(1));
 		return *this;
 	}
 
@@ -111,18 +296,27 @@ struct ScoreVector<int8_t, DELTA>
 		return ScoreVector(_mm256_cmpeq_epi8(data_, v.data_));
 	}
 
+	ScoreVector operator>(const ScoreVector& v) const {
+		return ScoreVector(_mm256_cmpgt_epi8(data_, v.data_));
+	}
+
 	friend uint32_t cmp_mask(const ScoreVector&v, const ScoreVector&w) {
 		return (uint32_t)_mm256_movemask_epi8(_mm256_cmpeq_epi8(v.data_, w.data_));
 	}
 
 	int operator [](unsigned i) const
 	{
-		return *(((uint8_t*)&data_) + i);
+		return *(((int8_t*)&data_) + i);
 	}
 
-	ScoreVector& set(unsigned i, uint8_t v)
+	ScoreVector& set(unsigned i, int8_t v)
 	{
-		*(((uint8_t*)&data_) + i) = v;
+		//*(((uint8_t*)&data_) + i) = v;
+		//data_ = _mm256_insert_epi8(data_, v, i);
+		alignas(32) std::array<int8_t, 32> s;
+		_mm256_store_si256((__m256i *)s.data(), data_);
+		s[i] = v;
+		data_ = _mm256_load_si256((__m256i*)s.data());
 		return *this;
 	}
 
@@ -153,6 +347,11 @@ struct ScoreVector<int8_t, DELTA>
 		_mm256_storeu_si256((__m256i*)ptr, data_);
 	}
 
+	void store_aligned(int8_t* ptr) const
+	{
+		_mm256_store_si256((__m256i*)ptr, data_);
+	}
+
 	friend std::ostream& operator<<(std::ostream& s, ScoreVector v)
 	{
 		int8_t x[32];
@@ -162,12 +361,39 @@ struct ScoreVector<int8_t, DELTA>
 		return s;
 	}
 
+	static ScoreVector load_aligned(const int8_t* x) {
+		return ScoreVector(_mm256_load_si256((const __m256i*)x));
+	}
+
 	void expand_from_8bit() {}
 
 	__m256i data_;
 
 };
 
+template<int i, int DELTA>
+static inline int8_t extract(ScoreVector<int8_t, DELTA> sv) {
+	return (int8_t)_mm256_extract_epi8(sv.data_, i);
+}
+
+template<int DELTA>
+static inline void store_expanded(ScoreVector<int8_t, DELTA> sv, int16_t* dst) {
+	const __m256i z = _mm256_setzero_si256();
+	const __m256i a = _mm256_permute4x64_epi64(sv.data_, 216);
+	__m256i b = _mm256_unpacklo_epi8(a, z);
+	__m256i c = _mm256_slli_si256(_mm256_cmpgt_epi8(z, b), 1);
+	_mm256_store_si256((__m256i*)dst, _mm256_or_si256(b, c));
+
+	b = _mm256_unpackhi_epi8(a, z);
+	c = _mm256_slli_si256(_mm256_cmpgt_epi8(z, b), 1);
+	_mm256_store_si256((__m256i*)(dst + 16), _mm256_or_si256(b, c));
+}
+
+template<int DELTA>
+static inline void store_expanded(ScoreVector<int8_t, DELTA> sv, int8_t* dst) {
+	_mm256_store_si256((__m256i*)dst, sv.data_);
+}
+
 template<int DELTA>
 struct ScoreTraits<ScoreVector<int8_t, DELTA>>
 {
@@ -215,7 +441,7 @@ struct ScoreVector<int8_t, DELTA>
 {
 
 	ScoreVector():
-		data_(::SIMD::_mm_set1_epi8(DELTA))
+		data_(_mm_set1_epi8(DELTA))
 	{}
 
 	explicit ScoreVector(__m128i data):
@@ -223,11 +449,11 @@ struct ScoreVector<int8_t, DELTA>
 	{}
 
 	explicit ScoreVector(int8_t x):
-		data_(::SIMD::_mm_set1_epi8(x))
+		data_(_mm_set1_epi8(x))
 	{}
 
 	explicit ScoreVector(int x):
-		data_(::SIMD::_mm_set1_epi8(x))
+		data_(_mm_set1_epi8(x))
 	{}
 
 	explicit ScoreVector(const int8_t* s) :
@@ -243,9 +469,11 @@ struct ScoreVector<int8_t, DELTA>
 	{
 		const __m128i* row = reinterpret_cast<const __m128i*>(&score_matrix.matrix8()[a << 5]);
 
-		__m128i high_mask = _mm_slli_epi16(_mm_and_si128(seq, ::SIMD::_mm_set1_epi8('\x10')), 3);
+		seq = letter_mask(seq);
+
+		__m128i high_mask = _mm_slli_epi16(_mm_and_si128(seq, _mm_set1_epi8('\x10')), 3);
 		__m128i seq_low = _mm_or_si128(seq, high_mask);
-		__m128i seq_high = _mm_or_si128(seq, _mm_xor_si128(high_mask, ::SIMD::_mm_set1_epi8('\x80')));
+		__m128i seq_high = _mm_or_si128(seq, _mm_xor_si128(high_mask, _mm_set1_epi8('\x80')));
 
 		__m128i r1 = _mm_load_si128(row);
 		__m128i r2 = _mm_load_si128(row + 1);
@@ -282,7 +510,7 @@ struct ScoreVector<int8_t, DELTA>
 	}
 
 	ScoreVector& operator++() {
-		data_ = _mm_adds_epi8(data_, ::SIMD::_mm_set1_epi8(1));
+		data_ = _mm_adds_epi8(data_, _mm_set1_epi8(1));
 		return *this;
 	}
 
@@ -294,6 +522,10 @@ struct ScoreVector<int8_t, DELTA>
 		return ScoreVector(_mm_cmpeq_epi8(data_, v.data_));
 	}
 
+	ScoreVector operator>(const ScoreVector& v) const {
+		return ScoreVector(_mm_cmpgt_epi8(data_, v.data_));
+	}
+
 	friend uint32_t cmp_mask(const ScoreVector&v, const ScoreVector&w) {
 		return _mm_movemask_epi8(_mm_cmpeq_epi8(v.data_, w.data_));
 	}
@@ -336,6 +568,11 @@ struct ScoreVector<int8_t, DELTA>
 		_mm_storeu_si128((__m128i*)ptr, data_);
 	}
 
+	void store_aligned(int8_t* ptr) const
+	{
+		_mm_store_si128((__m128i*)ptr, data_);
+	}
+
 	friend std::ostream& operator<<(std::ostream &s, ScoreVector v)
 	{
 		int8_t x[16];
@@ -345,12 +582,21 @@ struct ScoreVector<int8_t, DELTA>
 		return s;
 	}
 
+	static ScoreVector load_aligned(const int8_t* x) {
+		return ScoreVector(_mm_load_si128((const __m128i*)x));
+	}
+
 	void expand_from_8bit() {}
 
 	__m128i data_;
 
 };
 
+template<int i, int DELTA>
+static inline int8_t extract(ScoreVector<int8_t, DELTA> sv) {
+	return 0;
+}
+
 template<int DELTA>
 struct ScoreTraits<ScoreVector<int8_t, DELTA>>
 {
@@ -403,8 +649,9 @@ static inline int8_t extract_channel(const DISPATCH_ARCH::ScoreVector<int8_t, DE
 }
 
 template<int DELTA>
-static inline DISPATCH_ARCH::ScoreVector<int8_t, DELTA> set_channel(const DISPATCH_ARCH::ScoreVector<int8_t, DELTA>& v, const int i, const int8_t x) {
-	return DISPATCH_ARCH::ScoreVector<int8_t, DELTA>(v).set(i, x);
+static inline void set_channel(DISPATCH_ARCH::ScoreVector<int8_t, DELTA>& v, const int i, const int8_t x) {
+	v.set(i, x);
 }
 
+
 #endif
\ No newline at end of file
