@@ -68,6 +68,7 @@ void hash_seqs();
 void list_seeds();
 void prep_db();
 void greedy_vertex_cover();
+void merge_daa();
 #ifdef EXTRA
 void index_fasta();
 void fetch_seq();
@@ -89,6 +90,7 @@ namespace Incremental {
 int main(int ac, const char* av[])
 {
 	try {
+		init_motif_table();
 		CommandLineParser parser;
 		config = Config(ac, av, true, parser);
 
@@ -142,6 +144,7 @@ int main(int ac, const char* av[])
 			break;
 		case Config::cluster:
 		case Config::DEEPCLUST:
+		case Config::LINCLUST:
 			// Why is cluster_similarity not set at the end of the Config constructor?
 			if(!config.cluster_similarity.empty()){
 				string expression = RecursiveParser::clean_expression(&config.cluster_similarity);
@@ -216,6 +219,9 @@ int main(int ac, const char* av[])
 		case Config::RECLUSTER:
 			Cluster::recluster();
 			break;
+		case Config::MERGE_DAA:
+			merge_daa();
+			break;
 #ifdef EXTRA
 		case Config::INDEX_FASTA:
 			index_fasta();
