@@ -20,18 +20,19 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #pragma once
+#include <memory>
 #include <vector>
+#include <list>
 #include <map>
 #include <string>
 #include <assert.h>
 #include <stdlib.h>
 #include <stdexcept>
 #include <stdint.h>
-#include "ptr_vector.h"
 #include "options/option.h"
 
-template<typename _t>
-inline void read_option(_t &dst, const std::vector<std::string> &v)
+template<typename T>
+inline void read_option(T &dst, const std::vector<std::string> &v)
 {
 	throw std::runtime_error("Invalid option type.");
 }
@@ -62,6 +63,12 @@ inline void read_option<long long>(long long& dst, const std::vector<std::string
 	dst = atoll(v[0].c_str());
 }
 
+template<>
+inline void read_option<Option<int64_t>>(Option<int64_t>& dst, const std::vector<std::string>& v)
+{
+	dst = atoll(v[0].c_str());
+}
+
 template<>
 inline void read_option<unsigned long long>(unsigned long long& dst, const std::vector<std::string>& v)
 {
@@ -74,10 +81,23 @@ inline void read_option<double>(double &dst, const std::vector<std::string> &v)
 	dst = atof(v[0].c_str());
 }
 
+
+template<>
+inline void read_option<Option<double>>(Option<double> &dst, const std::vector<std::string> &v)
+{
+	dst = atof(v[0].c_str());
+}
+
 template<>
 inline void read_option<std::string>(std::string &dst, const std::vector<std::string> &v)
 { dst = v[0]; }
 
+template<>
+inline void read_option<Option<std::string>>(Option<std::string> &dst, const std::vector<std::string> &v)
+{
+	dst = v[0];
+}
+
 template<>
 inline void read_option<std::vector<std::string>>(std::vector<std::string> &dst, const std::vector<std::string> &v)
 {
@@ -94,7 +114,7 @@ template<>
 inline void read_option<bool>(bool &dst, const std::vector<std::string> &v)
 { dst = true; }
 
-template<typename _t>
+template<typename T>
 inline bool check_pcount(const std::vector<std::string> &v, const int min_count)
 {
 	return v.size() == 1;
@@ -118,92 +138,120 @@ inline bool check_pcount<Option<std::vector<std::string>>>(const std::vector<std
 	return v.size() >= (size_t)min_count;
 }
 
+template<typename T>
+inline bool check_present(const T& v) {
+	return false;
+}
 
-struct Option_base
-{
-	Option_base(const std::string &id, char short_id, const std::string &desc, bool disabled) :
-		id(id),
-		desc(desc),
-		short_id (short_id),
-		disabled(disabled)
-	{}
-	virtual void read(const std::vector<std::string> &v) = 0;
-	virtual void set_default() = 0;
-	virtual ~Option_base()
-	{}
-	const std::string id, desc;
-	const char short_id;
-	bool disabled;
-};
+template<>
+inline bool check_present<std::string>(const std::string& v) {
+	return !v.empty();
+}
 
-template<typename _t>
-struct OptionDesc : public Option_base
+template<typename T>
+inline void set_base_ptr(T& v, const OptionBase* ptr) {
+}
+
+template<typename T>
+inline void set_base_ptr(Option<T>& v, const OptionBase* ptr) {
+	v.base_ = ptr;
+}
+
+template<typename T>
+inline void set_option_default(T& option, const T& value) {
+	option = value;
+}
+
+template<typename T>
+inline void set_option_default(Option<T>& option, const Option<T>& value) {
+}
+
+template<typename T>
+struct OptionDesc : public OptionBase
 {
-	OptionDesc(const char *id, char short_id, const char *desc, bool disabled, _t &store, _t def, const int min_count) :
-		Option_base(id, short_id, desc, disabled),
+	OptionDesc(const char *id, char short_id, const char *desc, bool disabled, T &store, T def, const int min_count, const OptionsGroup* group) :
+		OptionBase(id, short_id, desc, disabled, group),
 		default_(def),
 		min_count(min_count),
-		store_(store)
+		store_(&store)
 	{ }
 	virtual void read(const std::vector<std::string> &v)
 	{
-		if (!check_pcount<_t>(v, min_count))
+		if (!check_pcount<T>(v, min_count))
 			throw std::runtime_error("Invalid parameter count for option '--" + id + "'");
-		read_option(store_, v);
+		read_option(*store_, v);
+	}
+	virtual bool present() {
+		return check_present<T>(*store_);			
 	}
 	virtual void set_default()
 	{
-		store_ = default_;
+		set_option_default(*store_, default_);
 	}
 	virtual ~OptionDesc()
 	{}
 private:
-	const _t default_;
+	const T default_;
 	const int min_count;
-	_t &store_;
+	T* store_;
 };
 
-struct Options_group
+struct CommandLineParserBase
+{
+	std::map<std::string, OptionBase*> map_;
+	std::map<char, OptionBase*> map_short_;
+};
+
+struct OptionsGroup
 {
 	struct Add_f
 	{
-		Add_f(Options_group &parent):
-			parent_(parent)
+		Add_f(OptionsGroup& parent) :
+			parent_(&parent)
 		{}
-		template<typename _t>
-		Add_f& operator()(const char *id, char short_id, const char *desc, _t &store, _t def = _t(), const int min_count = 1)
+		template<typename T>
+		Add_f& operator()(const char* id, char short_id, const char* desc, T& store, T def = T(), const int min_count = 1)
 		{
-			parent_.options.push_back(new OptionDesc<_t>(id, short_id, desc, parent_.disabled, store, def, min_count));
+			auto o = new OptionDesc<T>(id, short_id, desc, parent_->disabled, store, def, min_count, parent_);
+			parent_->options.emplace_back(o);
+			parent_->parent_->map_[id] = o;
+			parent_->parent_->map_short_[short_id] = o;
+			set_base_ptr(store, o);
 			return *this;
 		}
 	private:
-		Options_group &parent_;
+		OptionsGroup* parent_;
 	};
 	Add_f add()
 	{
 		return Add_f(*this);
 	}
-	Options_group(const char *title, bool disabled = false):
-		title (title),
-		disabled(disabled)
+	OptionsGroup(const char* title, const std::vector<unsigned>& commands, bool disabled, CommandLineParserBase* parent) :
+		title(title),
+		commands(commands),
+		disabled(disabled),
+		parent_(parent)
 	{}
-	PtrVector<Option_base> options;
+	std::list<std::unique_ptr<OptionBase>> options;
 	std::string title;
+	const std::vector<unsigned> commands;
 	bool disabled;
+private:
+	CommandLineParserBase* parent_;
 };
 
-struct Command_line_parser
+struct CommandLineParser : CommandLineParserBase
 {
-	Command_line_parser& add(const Options_group &group);
-	Command_line_parser& add_command(const char *s, const char *desc, unsigned code);
+	CommandLineParser& add_command(const char *s, const char *desc, unsigned code);
 	void store(int count, const char **str, unsigned &command);
+	void require(const char* option);
 	void print_help();
-private:
-	void store_option(const std::vector<std::string> &v);
+	OptionsGroup& add_group(const char *title, const std::vector<unsigned>& commands, bool disabled = false);
 
-	std::map<std::string, Option_base*> map_;
-	std::map<char, Option_base*> map_short_;
-	std::vector<const Options_group*> groups_;
+private:
+	void store_option(const std::vector<std::string> &v, unsigned command);
+
+	std::list<OptionsGroup> groups_;
 	std::map<std::string, unsigned> command_codes_;
 	std::vector<std::pair<std::string, std::string>> commands_;
 };
\ No newline at end of file
