@@ -2,19 +2,20 @@
 #include <utility>
 #include <assert.h>
 #include <limits.h>
+#include <array>
 
-template<int _n, typename _t>
+template<int N, typename T>
 struct RangePartition {
 
 	RangePartition(const int *begin, int count, int end) :
 		count_(1)
 	{
 		assert(count > 0);
-		std::pair<int, int> b[_n];
+		std::array<std::pair<int, int>, N> b;
 		for (int i = 0; i < count; ++i)
 			b[i] = { begin[i], i };
-		std::sort(b, b + count);
-		std::fill(mask_[0], mask_[0] + _n, std::numeric_limits<_t>::min());
+		std::sort(b.begin(), b.begin() + count);
+		std::fill(mask_[0], mask_[0] + N, std::numeric_limits<T>::min());
 		mask_[0][b[0].second] = 0;
 #ifdef DP_STAT
 		bit_mask_[0] = 1llu << b[0].second;
@@ -23,7 +24,7 @@ struct RangePartition {
 		for (int i = 1; i < count; ++i) {
 			if (begin_[count_ - 1] < b[i].first) {
 				begin_[count_] = b[i].first;
-				std::copy(mask_[count_ - 1], mask_[count_ - 1] + _n, mask_[count_]);
+				std::copy(mask_[count_ - 1], mask_[count_ - 1] + N, mask_[count_]);
 				mask_[count_][b[i].second] = 0;
 #ifdef DP_STAT
 				bit_mask_[count_] = bit_mask_[count_ - 1] | (1llu << b[i].second);
@@ -52,7 +53,7 @@ struct RangePartition {
 		return count_;
 	}
 
-	const _t* mask(int i) const {
+	const T* mask(int i) const {
 		return mask_[i];
 	}
 
@@ -64,10 +65,10 @@ struct RangePartition {
 
 private:
 
-	int begin_[_n + 1];
-	_t mask_[_n][_n];
+	int begin_[N + 1];
+	T mask_[N][N];
 #ifdef DP_STAT
-	uint64_t bit_mask_[_n];
+	uint64_t bit_mask_[N];
 #endif
 	int count_;
 	
