@@ -16,9 +16,7 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef ESCAPE_SEQUENCES_H_
-#define ESCAPE_SEQUENCES_H_
-
+#pragma once
 #include <limits.h>
 #include "text_buffer.h"
 
@@ -93,6 +91,4 @@ inline void print_escaped(TextBuffer& buf, const std::string& s, const EscapeSeq
 		esc->escape(s, tmp);
 		buf << tmp;
 	}
-}
-
-#endif
\ No newline at end of file
+}
\ No newline at end of file
