@@ -16,15 +16,10 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef COMPRESSED_STREAM_H_
-#define COMPRESSED_STREAM_H_
-
+#pragma once
 #include <string>
 #include <zlib.h>
 #include "stream_entity.h"
-#include "../util.h"
-
-using std::string;
 
 struct ZlibSource : public StreamEntity
 {
@@ -48,6 +43,4 @@ private:
 	void deflate_loop(const char *ptr, size_t count, int code);
 	static const size_t chunk_size = 1llu << 20;
 	z_stream strm;
-};
-
-#endif
\ No newline at end of file
+};
\ No newline at end of file
