@@ -16,22 +16,20 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef FILE_SOURCE_H_
-#define FILE_SOURCE_H_
-
+#pragma once
 #include "stream_entity.h"
 
 struct FileSource : public StreamEntity
 {
-	FileSource(const string &file_name);
-	FileSource(const string &file_name, FILE *file);
+	FileSource(const std::string &file_name);
+	FileSource(const std::string &file_name, FILE *file);
 	virtual void rewind() override;
-	virtual void seek(size_t pos) override;
+	virtual void seek(int64_t p, int origin) override;
 	virtual void seek_forward(size_t n) override;
-	virtual size_t tell() override;
+	virtual int64_t tell() override;
 	virtual size_t read(char *ptr, size_t count) override;
 	virtual void close() override;
-	virtual const string& file_name() const override
+	virtual const std::string& file_name() const override
 	{
 		return file_name_;
 	}
@@ -44,7 +42,5 @@ struct FileSource : public StreamEntity
 	{}
 protected:
 	FILE *f_;
-	const string file_name_;
-};
-
-#endif
\ No newline at end of file
+	const std::string file_name_;
+};
\ No newline at end of file
