@@ -16,16 +16,11 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef STREAM_ENTITY_H_
-#define STREAM_ENTITY_H_
-
+#pragma once
 #include <string>
 #include <utility>
 #include "exceptions.h"
 
-using std::string;
-using std::pair;
-
 struct StreamEntity
 {
 	StreamEntity(bool seekable = false):
@@ -40,7 +35,7 @@ struct StreamEntity
 	{
 		throw UnsupportedOperation();
 	}
-	virtual void seek(size_t pos)
+	virtual void seek(int64_t p, int origin)
 	{
 		throw UnsupportedOperation();
 	}
@@ -48,7 +43,7 @@ struct StreamEntity
 	{
 		throw UnsupportedOperation();
 	}
-	virtual size_t tell()
+	virtual int64_t tell()
 	{
 		throw UnsupportedOperation();
 	}
@@ -56,7 +51,7 @@ struct StreamEntity
 	{
 		throw UnsupportedOperation();
 	}
-	virtual pair<const char*, const char*> read()
+	virtual std::pair<const char*, const char*> read()
 	{
 		throw UnsupportedOperation();
 	}
@@ -64,7 +59,7 @@ struct StreamEntity
 	{
 		prev_->close();
 	}
-	virtual const string& file_name() const
+	virtual const std::string& file_name() const
 	{
 		return prev_->file_name();
 	}
@@ -72,7 +67,7 @@ struct StreamEntity
 	{
 		throw UnsupportedOperation();
 	}
-	virtual pair<char*, char*> write_buffer()
+	virtual std::pair<char*, char*> write_buffer()
 	{
 		throw UnsupportedOperation();
 	}
@@ -103,5 +98,3 @@ protected:
 	StreamEntity *prev_;
 	bool seekable_;
 };
-
-#endif
\ No newline at end of file
