@@ -27,4 +27,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #if ARCH_ID == 2
 #include "transpose32x32.h"
+#endif
+
+#if ARCH_ID == 3
+static inline void transpose(const signed char** data, size_t n, signed char* out, const __m512i&) {}
 #endif
\ No newline at end of file
